#!/usr/bin/env python
import socket
import subprocess
import httpx
import json
import codefast as cf
from codefast.utils import timeout
import os
import argparse
from .utils import get_system_info


class FastPing(argparse.ArgumentParser):
    def __init__(self):
        super().__init__(description='Ping a host.')
        self.add_argument('host', type=str, help='hostname or IP address')
        self.add_argument('-c', '--count', type=int, default=9,
                          help='number of packets to send')

    @property
    def os(self):
        return get_system_info()['system']

    @property
    def cmd(self):
        args = self.parse_args()
        os_name = self.os.lower()
        if 'darwin' in os_name:
            return "ping -c {} -i 0.1 -W 1 {}".format(args.count, args.host)
        elif 'linux' in os_name:
            return "ping -c {} -4 -i 0.1 -W 1 {}".format(args.count, args.host)

    @staticmethod
    def entrypoint():
        response = os.system(FastPing().cmd)
        return response == 0


class Cufo(object):
    @staticmethod
    def entrypoint():
        return os.system(
            "curl cufo.cc"
        )


class ProxyCheck(argparse.ArgumentParser):
    def __init__(self):
        super().__init__(description='Check if a proxy is working.')
        self.add_argument('host', type=str, help='hostname or IP address')
        self.add_argument('port', type=int, help='port number')
        self.args = self.parse_args()

    def is_reachable(self):
        try:
            socket.gethostbyname(self.args.host)
            return True
        except socket.error:
            return False

    def is_port_open(self):
        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        sock.settimeout(5)
        try:
            sock.connect((self.args.host, self.args.port))
            sock.shutdown(socket.SHUT_RDWR)
            return True
        except:
            return False
        finally:
            sock.close()

    def is_socks5_working(self):
        proxies = {
            'http://': 'socks5://{}:{}'.format(self.args.host, self.args.port),
            'https://': 'socks5://{}:{}'.format(self.args.host, self.args.port)
        }

        @timeout(5)
        def _worker():
            with httpx.Client(proxies=proxies) as client:
                response = client.get('http://www.google.com')
                return response.status_code == 200

        try:
            return _worker()
        except Exception:
            return False

    def is_http_working(self):
        proxies = {'http://': 'http://{}:{}'.format(self.args.host, self.args.port),
                   'https://': 'http://{}:{}'.format(self.args.host, self.args.port)}
        with httpx.Client(proxies=proxies) as client:
            try:
                response = client.get('http://www.google.com')
                return response.status_code == 200
            except:
                return False

    @staticmethod
    def entrypoint():
        pc = ProxyCheck()
        msgs = [
            ("Is [HOST] reachable", pc.is_reachable),
            ("Is [PORT] open", pc.is_port_open),
            ("Is [SOCKS5] proxy working", pc.is_socks5_working),
            ("Is [HTTP] proxy working", pc.is_http_working)
        ]
        for msg, f in msgs:
            b = f()
            btext = cf.fp.green(b) if b else cf.fp.red(b)
            print(f"{msg:<{30}}: {btext}")


'''
优化这个脚本，模拟 curl 命令，支持 POST 请求，支持输出到文件。
1. 使用示例一：`jcurl http://httpbin.org/post key1=value1 key2=value2`
可能有多个 k, v 对，也可能没有，所以要动态解析参数。等同于 `
curl -X POST http://httpbin.org/post -d 'key1=value1&key2=value2'`
2. `jcurl http://httpbin.org/post key1=value1 key2=value2 -o output.png`
如果有 `-o` 参数这时候，输出到文件 output.png。
'''


class JsonCurl:
    def __init__(self):
        parser = argparse.ArgumentParser(
            description='Simulate curl for JSON APIs with POST support.')
        parser.add_argument('url', type=str, help='URL of the API')
        parser.add_argument('data', nargs='*', type=str,
                            help='Data for POST request in key=value format')
        parser.add_argument('-o', '--output', type=str,
                            help='Output file', default=None)
        self.args = parser.parse_args()

    def post(self):
        data = {}
        if self.args.data:
            for item in self.args.data:
                key, value = item.split('=', 1)
                data[key] = value
        cf.info("data: {}".format(data))
        cf.info("url: {}".format(self.args.url))
        cmd = 'curl -s -X POST -H "Content-Type: application/json" -d \'{}\' {}'.format(
            json.dumps(data), self.args.url)
        cf.info("cmd: {}".format(cmd))
        response = subprocess.check_output(cmd, shell=True)
        cf.info("response: {}".format(response))

    @staticmethod
    def entrypoint():
        JsonCurl().post()
